@@ -7,6 +7,7 @@ module Network.HTTP.Client.Connection
     , connectionDropTillBlankLine
     , dummyConnection
     , openSocketConnection
+    , openSocketConnectionSize
     , makeConnection
     ) where
 
@@ -95,9 +96,9 @@ makeConnection r w c = do
         , connectionClose = c
         }
 
-socketConnection :: Socket -> IO Connection
-socketConnection socket = makeConnection
-    (recv socket 4096)
+socketConnection :: Socket -> Int -> IO Connection
+socketConnection socket chunksize = makeConnection
+    (recv socket chunksize)
     (sendAll socket)
     (sClose socket)
 
@@ -106,7 +107,15 @@ openSocketConnection :: (Socket -> IO ())
                      -> String -- ^ host
                      -> Int -- ^ port
                      -> IO Connection
-openSocketConnection tweakSocket hostAddress host port = do
+openSocketConnection f = openSocketConnectionSize f 8192
+
+openSocketConnectionSize :: (Socket -> IO ())
+                         -> Int -- ^ chunk size
+                         -> Maybe HostAddress
+                         -> String -- ^ host
+                         -> Int -- ^ port
+                         -> IO Connection
+openSocketConnectionSize tweakSocket chunksize hostAddress host port = do
     let hints = NS.defaultHints {
                           NS.addrFlags = [NS.AI_ADDRCONFIG]
                         , NS.addrSocketType = NS.Stream
@@ -134,7 +143,7 @@ openSocketConnection tweakSocket hostAddress host port = do
                 NS.setSocketOption sock NS.NoDelay 1
                 NS.connect sock (NS.addrAddress addr)
                 tweakSocket sock
-                socketConnection sock)
+                socketConnection sock chunksize)
 
 firstSuccessful :: [NS.AddrInfo] -> (NS.AddrInfo -> IO a) -> IO a
 firstSuccessful []     _  = error "getAddrInfo returned empty list"
