@@ -71,7 +71,8 @@ httpRaw
      :: Request
      -> Manager
      -> IO (Response BodyReader)
-httpRaw req' m = do
+httpRaw req0 m = do
+    req' <- mModifyRequest m $ mSetProxy m req0
     (req, cookie_jar') <- case cookieJar req' of
         Just cj -> do
             now <- getCurrentTime
@@ -88,9 +89,9 @@ httpRaw req' m = do
     -- connections after accepting the request headers, so we need to check for
     -- exceptions in both.
     ex <- try $ do
-        requestBuilder req ci
+        cont <- requestBuilder (dropProxyAuthSecure req) ci
 
-        getResponse connRelease timeout' req ci
+        getResponse connRelease timeout' req ci cont
 
     case (ex, isManaged) of
         -- Connection was reused, and might have been closed. Try again
