@@ -25,6 +25,8 @@ module Network.HTTP.Client.Manager
 #if !MIN_VERSION_base(4,6,0)
 import Prelude hiding (catch)
 #endif
+import Control.Applicative ((<|>))
+import Control.Arrow (first)
 import Data.Monoid (mappend)
 import System.IO (hClose, hFlush, IOMode(..))
 import qualified Data.IORef as I
@@ -160,9 +162,8 @@ addToList now maxCount x l@(Cons _ currCount _ _)
     | maxCount > currCount = (Cons x (currCount + 1) now l, Nothing)
     | otherwise = (l, Just x)
 
--- | Create a 'Manager'. You may manually call 'closeManager' to shut it down,
--- or allow the @Manager@ to be shut down automatically based on garbage
--- collection.
+-- | Create a 'Manager'. The @Manager@ will be shut down automatically via
+-- garbage collection.
 --
 -- Creating a new 'Manager' is a relatively expensive operation, you are
 -- advised to share a single 'Manager' between requests instead.
@@ -304,7 +305,8 @@ neFromList xs =
 --
 -- Since 0.1.0
 closeManager :: Manager -> IO ()
-closeManager = closeManager' . mConns
+closeManager _ = return ()
+{-# DEPRECATED closeManager "Manager will be closed for you automatically when no longer in use" #-}
 
 closeManager' :: I.IORef ConnsMap
               -> IO ()
@@ -318,7 +320,8 @@ closeManager' connsRef = mask_ $ do
 --
 -- Since 0.2.1
 withManager :: ManagerSettings -> (Manager -> IO a) -> IO a
-withManager settings = bracket (newManager settings) closeManager
+withManager settings f = newManager settings >>= f
+{-# DEPRECATED withManager "Use newManager instead" #-}
 
 safeConnClose :: Connection -> IO ()
 safeConnClose ci = connectionClose ci `catch` \(_ :: IOException) -> return ()
@@ -502,7 +505,8 @@ data EnvHelper = EHFromRequest
 envHelper :: Text -> EnvHelper -> IO (Request -> Request)
 envHelper name eh = do
     env <- getEnvironment
-    case lookup (T.unpack name) env of
+    let lenv = Map.fromList $ map (first $ T.toLower . T.pack) env
+    case lookup (T.unpack name) env <|> Map.lookup name lenv of
         Nothing  -> return noEnvProxy
         Just ""  -> return noEnvProxy
         Just str -> do
