@@ -27,6 +27,8 @@ module Network.HTTP.Client.Types
     , NonEmptyList (..)
     , ConnHost (..)
     , ConnKey (..)
+    , ProxyOverride (..)
+    , StreamFileStatus (..)
     ) where
 
 import qualified Data.Typeable as T (Typeable)
@@ -40,6 +42,7 @@ import Data.Int (Int64)
 import Data.Default.Class
 import Data.Foldable (Foldable)
 import Data.Monoid
+import Data.String (IsString, fromString)
 import Data.Time (UTCTime)
 import Data.Traversable (Traversable)
 import qualified Data.List as DL
@@ -50,6 +53,7 @@ import qualified Data.IORef as I
 import qualified Data.Map as Map
 import Data.Text (Text)
 import Data.Streaming.Zlib (ZlibException)
+import Control.Concurrent.MVar (MVar)
 
 -- | An @IO@ action that represents an incoming response body coming from the
 -- server. Data provided by this action has already been gunzipped and
@@ -84,7 +88,17 @@ data HttpException = StatusCodeException Status ResponseHeaders CookieJar
                    | HandshakeFailed
                    | OverlongHeaders
                    | ResponseTimeout
-                   | FailedConnectionException String Int -- ^ host/port
+                   | FailedConnectionException String Int
+                   -- ^ host/port
+                   --
+                   -- Note that in old versions of http-client and
+                   -- http-conduit, this exception would indicate a failed
+                   -- attempt to create a connection. However, since (at least)
+                   -- http-client 0.4, it indicates a timeout occurred while
+                   -- trying to establish the connection. For more information
+                   -- on this, see:
+                   --
+                   -- <https://github.com/snoyberg/http-client/commit/b86b1cdd91e56ee33150433dedb32954d2082621#commitcomment-10718689>
                    | FailedConnectionException2 String Int Bool SomeException -- ^ host/port/secure
                    | ExpectedBlankAfter100Continue
                    | InvalidStatusLine S.ByteString
@@ -108,6 +122,15 @@ data HttpException = StatusCodeException Status ResponseHeaders CookieJar
                    -- ^
                    --
                    -- Since 0.3
+                   | InvalidProxyEnvironmentVariable Text Text
+                   -- ^ Environment name and value
+                   --
+                   -- Since 0.4.7
+                   | ResponseLengthAndChunkingBothUsed
+                   -- ^ Detect a case where both the @content-length@ header
+                   -- and @transfer-encoding: chunked@ are used. Since 0.4.8.
+                   --
+                   -- Since 0.4.11 this exception isn't thrown anymore.
     deriving (Show, T.Typeable)
 instance Exception HttpException
 
@@ -185,6 +208,11 @@ data RequestBody
     | RequestBodyStream Int64 (GivesPopper ())
     | RequestBodyStreamChunked (GivesPopper ())
     deriving T.Typeable
+-- |
+--
+-- Since 0.4.12
+instance IsString RequestBody where
+    fromString str = RequestBodyBS (fromString str)
 instance Monoid RequestBody where
     mempty = RequestBodyBS S.empty
     mappend x0 y0 =
@@ -389,6 +417,20 @@ data Request = Request
     -- empty.
     --
     -- Since 0.1.0
+
+    , requestVersion :: HttpVersion
+    -- ^ HTTP version to send to server.
+    --
+    -- Default: HTTP 1.1
+    --
+    -- Since 0.4.3
+
+    , onRequestBodyException :: SomeException -> IO ()
+    -- ^ How to deal with exceptions thrown while sending the request.
+    --
+    -- Default: ignore @IOException@s, rethrow all other exceptions.
+    --
+    -- Since: 0.4.6
     }
     deriving T.Typeable
 
@@ -494,6 +536,32 @@ data ManagerSettings = ManagerSettings
     -- Default: 512
     --
     -- Since 0.3.7
+    , managerModifyRequest :: Request -> IO Request
+    -- ^ Perform the given modification to a @Request@ before performing it.
+    --
+    -- Default: no modification
+    --
+    -- Since 0.4.4
+    , managerProxyInsecure :: ProxyOverride
+    -- ^ How HTTP proxy server settings should be discovered.
+    --
+    -- Default: respect the @proxy@ value on the @Request@ itself.
+    --
+    -- Since 0.4.7
+    , managerProxySecure :: ProxyOverride
+    -- ^ How HTTPS proxy server settings should be discovered.
+    --
+    -- Default: respect the @proxy@ value on the @Request@ itself.
+    --
+    -- Since 0.4.7
+    }
+    deriving T.Typeable
+
+-- | How the HTTP proxy server settings should be discovered.
+--
+-- Since 0.4.7
+newtype ProxyOverride = ProxyOverride
+    { runProxyOverride :: Bool -> IO (Request -> Request)
     }
     deriving T.Typeable
 
@@ -505,6 +573,12 @@ data ManagerSettings = ManagerSettings
 data Manager = Manager
     { mConns :: I.IORef ConnsMap
     -- ^ @Nothing@ indicates that the manager is closed.
+    , mConnsBaton :: MVar ()
+    -- ^ Used to indicate to the reaper thread that it has some work to do.
+    -- This must be filled every time a connection is returned to the manager.
+    -- While redundant with the @IORef@ above, this allows us to have the
+    -- reaper thread fully blocked instead of running every 5 seconds when
+    -- there are no connections to manage.
     , mMaxConns :: Int
     -- ^ This is a per-@ConnKey@ value.
     , mResponseTimeout :: Maybe Int
@@ -515,6 +589,9 @@ data Manager = Manager
     , mRetryableException :: SomeException -> Bool
     , mWrapIOException :: forall a. IO a -> IO a
     , mIdleConnectionCount :: Int
+    , mModifyRequest :: Request -> IO Request
+    , mSetProxy :: Request -> Request
+    -- ^ See 'managerProxy'
     }
     deriving T.Typeable
 
@@ -535,5 +612,15 @@ data ConnHost =
 
 -- | @ConnKey@ consists of a hostname, a port and a @Bool@
 -- specifying whether to use SSL.
-data ConnKey = ConnKey ConnHost Int Bool
+data ConnKey = ConnKey ConnHost Int S.ByteString Int Bool
+    deriving (Eq, Show, Ord, T.Typeable)
+
+-- | Status of streaming a request body from a file.
+--
+-- Since 0.4.9
+data StreamFileStatus = StreamFileStatus
+    { fileSize :: Int64
+    , readSoFar :: Int64
+    , thisChunkSize :: Int
+    }
     deriving (Eq, Show, Ord, T.Typeable)
